/*=============================================================================
    FILE:           main.c

    DESCRIPTION:    CLD SC57x CDC/ACM example mainline file.

    Copyright (c) 2019 Closed Loop Design, LLC

    This software is supplied "AS IS" without any warranties, express, implied
    or statutory, including but not limited to the implied warranties of fitness
    for purpose, satisfactory quality and non-infringement. Closed Loop Design LLC
    extends you a royalty-free right to use, reproduce and distribute this source
    file as well as executable files created using this source file for use with
    Analog Devices SC5xx family processors only. Nothing else gives you
    the right to use this source file.

==============================================================================*/
/*!
 * @file      main.c
 * @brief     CLD SC57x CDC/ACM example mainline file.
 *
 * @details
 *            This file contains mainline state machine used to implement a
 *            CDC/ACM device using the cld_sc57x_cdc_lib library on
 *            a SC589 Cortex-A5 processor.
 *
 *            Copyright (c) 2018 Closed Loop Design, LLC
 *
 *            This software is supplied "AS IS" without any warranties, express, implied
 *            or statutory, including but not limited to the implied warranties of fitness
 *            for purpose, satisfactory quality and non-infringement. Closed Loop Design LLC
 *            extends you a royalty-free right to use, reproduce and distribute this source
 *            file as well as executable files created using this source file for use with
 *            Analog Devices SC5xx family processors only. Nothing else gives you
 *            the right to use this source file.
 *
 */

#include <sys/platform.h>
#include <sys/adi_core.h>
#include "adi_initialize.h"

#include "user_cdc.h"
#include <services/gpio/adi_gpio.h>
#include <services/pwr/adi_pwr.h>
#include <services/tmr/adi_tmr.h>

/* Core clock is generated from CGU0 */
#define CGU_DEV                 (0)             /*!< Clock Generation Unit device number. */
#define MHZ                     (1000000u)      /*!< Megahertz multiplier */
#define CLKIN                   (25u * MHZ)     /*!< 25MHz clock-in */
#define CORE_MAX                (450u * MHZ)    /*!< 450MHz Core Clock */
#define SYSCLK_MAX              (225u * MHZ)    /*!< 225 SysClk */
#define SCLK_MAX                (112u * MHZ)    /*!< 112 MHz SCLK */

#define DELAY_COUNT             (100000u)       /*!< Delay value used to reset the ADC/DAC. */

/**
 * Mainline state machine states.
 */
typedef enum
{
    MAIN_STATE_SYSTEM_INIT,                     /*!< Initialize the system */
    MAIN_STATE_USER_INIT,                       /*!< Initialize the User code */
    MAIN_STATE_RUN,                             /*!< Runtime state */
    MAIN_STATE_ERROR                            /*!< Error State */
} Main_States;

/* Gpio */
unsigned long GpioMemory[ADI_GPIO_CALLBACK_MEM_SIZE];   /*!< ADI System Services GPIO memory*/
unsigned long gpioMaxCallbacks;                         /*!< ADI System Services GPIO max callbacks variable*/
unsigned char TimerMemory[ADI_TMR_MEMORY];;

extern void ConfigSoftSwitches(void);
static unsigned long PowerServiceInit(void);
static unsigned long GpioInit(void);
static void timer_init (void);

/**
 * @brief Example project main() function.
 *
 * @details
 * Mainline Function for the project.
 */
int main()
{
    Main_States main_state = MAIN_STATE_SYSTEM_INIT;
    User_Init_Return_Code rv;

    /* Begin adding your custom code here */
    while (1)
    {
        switch (main_state)
        {
            case MAIN_STATE_SYSTEM_INIT:
                adi_initComponents(); /* auto-generated code */

                /**
                     * The default startup code does not include any functionality to allow
                     * core 0 to enable core 1 and core 2. A convenient way to enable
                     * core 1 and core 2 is to use the adi_core_enable function.
                     */
                adi_core_enable(ADI_CORE_SHARC0);
                adi_core_enable(ADI_CORE_SHARC1);

                if (PowerServiceInit() != 0)
                {
                    main_state = MAIN_STATE_ERROR;
                }
                else
                {
                    timer_init();

                    /* Software Switch Configuration for the EZ-BOARD */
                    ConfigSoftSwitches();

                    if (GpioInit() != 0)
                    {
                        main_state = MAIN_STATE_ERROR;
                    }
                    else
                    {
                        main_state = MAIN_STATE_USER_INIT;
                    }
                }
            break;
            case MAIN_STATE_USER_INIT:
                rv = user_cdc_init();
                if (rv == USER_CDC_INIT_SUCCESS)
                {
                    main_state = MAIN_STATE_RUN;
                }
                else if (rv == USER_CDC_INIT_FAILED)
                {
                    main_state = MAIN_STATE_ERROR;
                }
            break;

            case MAIN_STATE_RUN:
                 user_cdc_main();
            break;

            case MAIN_STATE_ERROR:

            break;
        }
    }

    return 0;
}


 /**
  * Initializes ADI power system service.
  * @return 0 = Success. 1 = Error
  */
static unsigned long PowerServiceInit(void)
{
    unsigned long Result = 0u;

    if((uint32_t)adi_pwr_Init(CGU_DEV, CLKIN) != 0u)
    {
        /* return error */
        return 1u;
    }
    if((uint32_t)adi_pwr_SetPowerMode(CGU_DEV, ADI_PWR_MODE_FULL_ON) != 0u)
    {
        /* return error */
        return 1u;
    }
    if((uint32_t)adi_pwr_SetFreq(CGU_DEV, CORE_MAX, SYSCLK_MAX) != 0u)
    {
        /* return error */
        return 1u;
    }

    return Result;
}


  /**
 * @brief Initializes GPIO service.
 *
 * @details
 * Initializes the ADI GPIO system service, and resets the ADAU1962A and ADAU1979 Codecs.
 * @return 0 = Success. 1 = Error
 */
static unsigned long GpioInit(void)
{
    unsigned long Result = 0u;
    /* Loop variable */
    volatile uint32_t i;

    if((unsigned long)adi_gpio_Init((void*)GpioMemory, ADI_GPIO_CALLBACK_MEM_SIZE, &gpioMaxCallbacks) != 0u)
    {
        /* return error */
        return 1u;
    }

#if defined(__ADSPBF707_FAMILY__) || defined(__ADSPSC589_FAMILY__)
    if((unsigned long)adi_gpio_SetDirection(ADI_GPIO_PORT_A, ADI_GPIO_PIN_14, ADI_GPIO_DIRECTION_OUTPUT) != 0u)
    {
        /* return error */
        return 1u;
    }
    /* bring reset low */
    if((unsigned long)adi_gpio_Clear(ADI_GPIO_PORT_A, ADI_GPIO_PIN_14) != 0u)
    {
        /* return error */
        return 1u;
    }

    /* delay */
    for (i = DELAY_COUNT; i ; i --);

    /* bring reset high */
    if((unsigned long)adi_gpio_Set(ADI_GPIO_PORT_A, ADI_GPIO_PIN_14) != 0u)
    {
        /* return error */
        return 1u;
    }

#elif defined(__ADSPSC573_FAMILY__)
    if((unsigned long)adi_gpio_SetDirection(ADI_GPIO_PORT_A, ADI_GPIO_PIN_6, ADI_GPIO_DIRECTION_OUTPUT) != 0u)
    {
        /* return error */
        return 1u;
    }

    if((unsigned long)adi_gpio_SetDirection(ADI_GPIO_PORT_A, ADI_GPIO_PIN_7, ADI_GPIO_DIRECTION_OUTPUT) != 0u)
    {
        /* return error */
        return 1u;
    }

    /* bring reset low */
    if((unsigned long)adi_gpio_Clear(ADI_GPIO_PORT_A, ADI_GPIO_PIN_6) != 0u)
    {
        /* return error */
        return 1u;
    }

    /* bring reset low */
    if((unsigned long)adi_gpio_Clear(ADI_GPIO_PORT_A, ADI_GPIO_PIN_7) != 0u)
    {
        /* return error */
        return 1u;
    }

    /* delay */
    for (i = DELAY_COUNT; i ; i --);

    /* bring reset high */
    if((uint32_t)adi_gpio_Set(ADI_GPIO_PORT_A, ADI_GPIO_PIN_6) != 0u)
    {
        /* return error */
        return 1u;
    }

    /* bring reset high */
    if((uint32_t)adi_gpio_Set(ADI_GPIO_PORT_A, ADI_GPIO_PIN_7) != 0u)
    {
        /* return error */
        return 1u;
    }

#else
#error "processor not supported"
#endif

    /* delay */
    for (i = DELAY_COUNT; i ; i --);

    return Result;
}

 /**
* @brief Initializes Timer service.
*
* @details
* Initializes the ADI timer system service, and configures the timer to
* generate a interrupt every 125us.
*/
static void timer_init (void)
{
    ADI_TMR_HANDLE   hTimer;
    ADI_TMR_RESULT   eTmrResult;

    /* Configure a 1ms Timer using the ADI system services */

    /* Open the timer */
    if( (eTmrResult = adi_tmr_Open (0,
                        TimerMemory,
                        ADI_TMR_MEMORY,
                        user_cdc_timer_handler,
                        0,
                        &hTimer)) != ADI_TMR_SUCCESS)
     {
         /* Failed to open the timer handle the error here */
     }

     /*
     * Use the GP timer's API's to configure and enable the timer
     *
     */

    /* Set the mode to PWM OUT */
    eTmrResult = adi_tmr_SetMode(hTimer, ADI_TMR_MODE_CONTINUOUS_PWMOUT);

    /* Set the IRQ mode to get interrupt after timer counts to Delay + Width */
    eTmrResult = adi_tmr_SetIRQMode(hTimer, ADI_TMR_IRQMODE_WIDTH_DELAY);

    /* Set the Period */
    eTmrResult = adi_tmr_SetPeriod(hTimer, 14000);

    /* Set the timer width */
    eTmrResult = adi_tmr_SetWidth(hTimer, 14000/2);

    /* Set the timer delay */
    eTmrResult = adi_tmr_SetDelay(hTimer, 100);

    /* Enable the timer */
    eTmrResult = adi_tmr_Enable(hTimer, true);
}
